@@ -32,6 +32,8 @@
 #include <glib-object.h>
 
 #include <libekolab/camel-kolab-imapx-store.h>
+#include <libekolab/e-source-kolab-folder.h>
+#include <libekolab/kolab-util-backend.h>
 
 #include <libekolabutil/kolab-util-error.h>
 #include <libekolabutil/kolab-util-folder.h>
@@ -80,6 +82,96 @@ kolab_folder_metadata_ui_create_folder_type_map (KolabFolderMetaUIData *uidata)
 	uidata->widgets->folder_type_map = map;
 }
 
+static void
+kolab_folder_metadata_ui_syncstrategy_fillbox (KolabFolderMetaUIData *uidata)
+{
+	GtkComboBoxText *box = NULL;
+	const gchar *strategy = NULL;
+	KolabSyncStrategyID ii = 0;
+
+	g_assert (uidata != NULL);
+	g_assert (uidata->widgets != NULL);
+
+	box = GTK_COMBO_BOX_TEXT (uidata->widgets->cbox_syncstrategy);
+
+	for (ii = 0; ii < KOLAB_SYNC_LAST_STRATEGY; ii++) {
+		strategy = kolab_util_backend_get_sync_strategy_desc (ii);
+		gtk_combo_box_text_insert_text (box,
+		                                (gint) ii,
+		                                strategy);
+	}
+
+	gtk_combo_box_set_active (GTK_COMBO_BOX (box),
+	                          KOLAB_SYNC_STRATEGY_DEFAULT);
+}
+
+static KolabSyncStrategyID
+kolab_folder_metadata_ui_syncstrategy_get (KolabFolderMetaUIData *uidata)
+{
+	KolabSyncStrategyID strategy = KOLAB_SYNC_STRATEGY_DEFAULT;
+	ESource *source = NULL;
+	ESourceResource *resource = NULL;
+	gboolean is_kolab = FALSE;
+
+	g_return_val_if_fail (uidata != NULL, KOLAB_SYNC_STRATEGY_DEFAULT);
+	g_return_val_if_fail (E_IS_SHELL_VIEW (uidata->shell_view), KOLAB_SYNC_STRATEGY_DEFAULT);
+
+	is_kolab = e_kolab_plugin_util_ui_get_selected_source (uidata->shell_view,
+	                                                       &source);
+	g_return_val_if_fail (is_kolab, KOLAB_SYNC_STRATEGY_DEFAULT);
+
+	resource = E_SOURCE_RESOURCE (e_source_get_extension (source,
+	                                                      E_SOURCE_EXTENSION_KOLAB_FOLDER));
+	strategy = e_source_kolab_folder_get_sync_strategy (E_SOURCE_KOLAB_FOLDER (resource));
+
+	g_object_unref (source);
+
+	return strategy;
+}
+
+static gboolean
+kolab_folder_metadata_ui_syncstrategy_set (KolabFolderMetaUIData *uidata,
+                                           GCancellable *cancellable,
+                                           GError **err)
+{
+	ESource *source = NULL;
+	ESourceResource *resource = NULL;
+	gboolean is_kolab = FALSE;
+	GError *tmp_err = NULL;
+	gboolean ok = TRUE;
+
+	g_return_val_if_fail (uidata != NULL, FALSE);
+	g_return_val_if_fail (E_IS_SHELL_VIEW (uidata->shell_view), FALSE);
+	/* cancellable may be NULL */
+	g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
+
+	is_kolab = e_kolab_plugin_util_ui_get_selected_source (uidata->shell_view,
+	                                                       &source);
+	if (! is_kolab) {
+		g_warning ("%s()[%u] expected Kolab source, got something else, skipping",
+		           __func__, __LINE__);
+		goto exit;
+	}
+
+	resource = E_SOURCE_RESOURCE (e_source_get_extension (source,
+	                                                      E_SOURCE_EXTENSION_KOLAB_FOLDER));
+	e_source_kolab_folder_set_sync_strategy (E_SOURCE_KOLAB_FOLDER (resource),
+	                                         uidata->metadata->strategy);
+	ok = e_source_write_sync (source,
+	                          cancellable,
+	                          &tmp_err);
+ exit:
+	if (tmp_err != NULL) {
+		g_propagate_error (err, tmp_err);
+		ok = FALSE;
+	}
+
+	if (source != NULL)
+		g_object_unref (source);
+
+	return ok;
+}
+
 static void
 kolab_folder_metadata_ui_foldertype_cb (GtkRadioButton *btn,
                                         gpointer userdata)
@@ -103,7 +195,7 @@ kolab_folder_metadata_ui_foldertype_cb (GtkRadioButton *btn,
 	if ((foldertype > KOLAB_FOLDER_TYPE_INVAL) &&
 	    (foldertype < KOLAB_FOLDER_LAST_TYPE)) {
 		uidata->metadata->foldertype = foldertype;
-		uidata->changed = TRUE;
+		uidata->changed_metadata = TRUE;
 	}
 
 	ok_btn = e_kolab_plugin_util_ui_dialog_ref_button (uidata->dialog,
@@ -114,6 +206,32 @@ kolab_folder_metadata_ui_foldertype_cb (GtkRadioButton *btn,
 	g_object_unref (ok_btn);
 }
 
+static void
+kolab_folder_metadata_ui_syncstrategy_cb (GtkComboBoxText *box,
+                                          gpointer userdata)
+{
+	GtkWidget *ok_btn = NULL;
+	KolabFolderMetaUIData *uidata = NULL;
+	gint active = 0;
+
+	g_return_if_fail (GTK_IS_COMBO_BOX_TEXT (box));
+	g_return_if_fail (userdata != NULL);
+
+	uidata = (KolabFolderMetaUIData *) userdata;
+	g_return_if_fail (uidata->metadata != NULL);
+
+	active = gtk_combo_box_get_active (GTK_COMBO_BOX (box));
+	uidata->metadata->strategy = (KolabFolderTypeID) active;
+	uidata->changed_syncstrategy = TRUE;
+
+	ok_btn = e_kolab_plugin_util_ui_dialog_ref_button (uidata->dialog,
+	                                                   GTK_STOCK_OK,
+	                                                   TRUE);
+	g_return_if_fail (GTK_IS_BUTTON (ok_btn));
+	gtk_widget_set_sensitive (ok_btn, TRUE);
+	g_object_unref (ok_btn);
+}
+
 static void
 kolab_folder_metadata_ui_show_all_cb (GtkToggleButton *btn,
                                       gpointer userdata)
@@ -128,6 +246,7 @@ kolab_folder_metadata_ui_show_all_cb (GtkToggleButton *btn,
 	g_return_if_fail (uidata->metadata != NULL);
 
 	uidata->metadata->show_all = !(uidata->metadata->show_all);
+	uidata->changed_visibility = TRUE;
 
 	ok_btn = e_kolab_plugin_util_ui_dialog_ref_button (uidata->dialog,
 	                                                   GTK_STOCK_OK,
@@ -156,11 +275,14 @@ e_kolab_folder_metadata_ui_new (void)
 
 	/* documenting initial settings */
 	uidata->shell_view = NULL;
+	uidata->shell_context = KOLAB_FOLDER_CONTEXT_INVAL;
 	uidata->alert_bar = NULL;
 	uidata->dialog = NULL;
 	uidata->foldername = NULL;
 	uidata->sourcename = NULL;
-	uidata->changed = FALSE;
+	uidata->changed_metadata = FALSE;
+	uidata->changed_syncstrategy = FALSE;
+	uidata->changed_visibility = FALSE;
 
 	uidata->widgets = g_new0 (KolabFolderMetaUIWidgets, 1);
 	uidata->metadata = kolab_data_folder_metadata_new ();
@@ -221,7 +343,7 @@ e_kolab_folder_metadata_ui_new (void)
 
 	kolab_folder_metadata_ui_create_folder_type_map (uidata);
 
-	/* "Show all folders" */
+	/* folder options */
 
 	widget = gtk_frame_new (C_("Kolab Folder Metadata",
 	                           "Kolab Folder Options"));
@@ -229,25 +351,37 @@ e_kolab_folder_metadata_ui_new (void)
 	gtk_container_add (GTK_CONTAINER (uidata->widgets->container), widget);
 	uidata->widgets->frame_options = widget;
 
+	grid = gtk_grid_new ();
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (grid), GTK_ORIENTATION_VERTICAL);
+	gtk_container_set_border_width (GTK_CONTAINER (grid), 4);
+	gtk_container_add (GTK_CONTAINER (uidata->widgets->frame_options), grid);
+
+	/* option: folder sync conflict resolution strategy */
+	widget = gtk_label_new (C_("Sync Conflict Resolution",
+	                           "Strategy for resolving PIM conflicts when synchronizing\n"
+	                           "with the Kolab server"));
+	gtk_misc_set_alignment (GTK_MISC (widget), 0.0f, 0.66f);
+	gtk_misc_set_padding (GTK_MISC (widget), 4, 4);
+	gtk_container_add (GTK_CONTAINER (grid), widget);
+	widget = gtk_combo_box_text_new ();
+	uidata->widgets->cbox_syncstrategy = widget;
+	gtk_container_add (GTK_CONTAINER (grid), uidata->widgets->cbox_syncstrategy);
+	kolab_folder_metadata_ui_syncstrategy_fillbox (uidata);
+
+	/* show all folders */
+
 	widget = gtk_check_button_new ();
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (widget), 6);
-	gtk_container_add (GTK_CONTAINER (uidata->widgets->frame_options), widget);
+	gtk_container_set_border_width (GTK_CONTAINER (widget), 4);
 	uidata->widgets->chk_btn_show_all = widget;
 	widget = gtk_label_new (C_("Kolab Folder Options",
 	                           "Show all PIM folders in this Kolab account"));
 	gtk_container_add (GTK_CONTAINER (uidata->widgets->chk_btn_show_all), widget);
+	gtk_container_add (GTK_CONTAINER (uidata->widgets->container), uidata->widgets->chk_btn_show_all);
 
-	/* "show all folders" switch */
-
-	uidata->widgets->chk_btn_show_all_handler_id =
-		g_signal_connect (G_OBJECT (uidata->widgets->chk_btn_show_all),
-		                  "toggled",
-		                  G_CALLBACK (kolab_folder_metadata_ui_show_all_cb),
-		                  uidata);
+	/* signals */
 
 	/* folder type radio buttons */
-
 	for (ii = 0; ii < KOLAB_FOLDER_META_UI_NUM_TYPES; ii++) {
 		uidata->widgets->radio_btn_handler_id[ii] =
 			g_signal_connect (G_OBJECT (uidata->widgets->radio_btn_type[ii]),
@@ -256,6 +390,19 @@ e_kolab_folder_metadata_ui_new (void)
 			                  uidata);
 	}
 
+	/* sync strategy combo box */
+	uidata->widgets->cbox_syncstrategy_id =
+		g_signal_connect (G_OBJECT (uidata->widgets->cbox_syncstrategy),
+		                  "changed",
+		                  G_CALLBACK (kolab_folder_metadata_ui_syncstrategy_cb),
+		                  uidata);
+
+	/* "show all folders" switch */
+	uidata->widgets->chk_btn_show_all_handler_id =
+		g_signal_connect (G_OBJECT (uidata->widgets->chk_btn_show_all),
+		                  "toggled",
+		                  G_CALLBACK (kolab_folder_metadata_ui_show_all_cb),
+		                  uidata);
 	return uidata;
 }
 
@@ -304,24 +451,6 @@ e_kolab_folder_metadata_ui_update_from_uidata (KolabFolderMetaUIData *uidata)
 	 * before the user actually uses it.
 	 */
 
-	/* whether to show all PIM folders in email view */
-
-	widget = uidata->widgets->chk_btn_show_all;
-	g_return_if_fail (GTK_IS_CHECK_BUTTON (widget));
-	handler_id = uidata->widgets->chk_btn_show_all_handler_id;
-
-	g_signal_handler_block (widget, handler_id);
-	if (uidata->metadata->show_all) {
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
-		                              TRUE);
-		g_debug ("%s: set toggle button active", __func__);
-	} else {
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
-		                              FALSE);
-		g_debug ("%s: set toggle button inactive", __func__);
-	}
-	g_signal_handler_unblock (widget, handler_id);
-
 	/* radio button group (current folder type) */
 	for (ii = 0; ii < KOLAB_FOLDER_META_UI_NUM_TYPES; ii++) {
 		g_signal_handler_block (uidata->widgets->radio_btn_type[ii],
@@ -362,6 +491,32 @@ e_kolab_folder_metadata_ui_update_from_uidata (KolabFolderMetaUIData *uidata)
 		g_signal_handler_unblock (uidata->widgets->radio_btn_type[ii],
 		                          uidata->widgets->radio_btn_handler_id[ii]);
 	}
+
+	/* PIM options */
+
+	/* option: sync conflict strategy */
+	widget = uidata->widgets->cbox_syncstrategy;
+	g_return_if_fail (GTK_IS_COMBO_BOX_TEXT (widget));
+	handler_id = uidata->widgets->cbox_syncstrategy_id;
+	g_signal_handler_block (widget, handler_id);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (widget),
+	                          (gint) uidata->metadata->strategy);
+	g_signal_handler_unblock (widget, handler_id);
+
+	/* whether to show all PIM folders in email view */
+
+	widget = uidata->widgets->chk_btn_show_all;
+	g_return_if_fail (GTK_IS_CHECK_BUTTON (widget));
+	handler_id = uidata->widgets->chk_btn_show_all_handler_id;
+
+	g_signal_handler_block (widget, handler_id);
+	if (uidata->metadata->show_all)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
+		                              TRUE);
+	else
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
+		                              FALSE);
+	g_signal_handler_unblock (widget, handler_id);
 }
 
 gboolean
@@ -370,9 +525,10 @@ e_kolab_folder_metadata_ui_query_store (KolabFolderMetaUIData *uidata,
                                         GError **err)
 {
 	CamelKolabIMAPXStore *kstore = NULL;
+	KolabSyncStrategyID strategy = KOLAB_SYNC_STRATEGY_DEFAULT;
 	gchar *selected_path = NULL;
 	GError *tmp_err = NULL;
-	gboolean ok = FALSE;
+	gboolean ok = TRUE;
 
 	g_return_val_if_fail (uidata != NULL, FALSE);
 	/* cancellable may be NULL */
@@ -381,6 +537,9 @@ e_kolab_folder_metadata_ui_query_store (KolabFolderMetaUIData *uidata,
 	g_return_val_if_fail (E_IS_SHELL_VIEW (uidata->shell_view), FALSE);
 	g_return_val_if_fail (uidata->metadata != NULL, FALSE);
 
+	if (uidata->shell_context != KOLAB_FOLDER_CONTEXT_EMAIL)
+		goto skip_email;
+
 	/* If we cannot get the store here, it means
 	 * that the store associated with the selected
 	 * path is not a CamelKolabIMAPXStore. This
@@ -406,7 +565,21 @@ e_kolab_folder_metadata_ui_query_store (KolabFolderMetaUIData *uidata,
 		                                         TRUE,
 		                                         cancellable,
 		                                         &tmp_err);
+ skip_email:
+
+	if (! ((uidata->shell_context == KOLAB_FOLDER_CONTEXT_CALENDAR) ||
+	       (uidata->shell_context == KOLAB_FOLDER_CONTEXT_CONTACT)))
+		goto exit;
+
+	uidata->metadata->foldername = g_strdup (uidata->foldername);
+	uidata->metadata->foldertype =
+		e_kolab_plugin_util_ui_get_shell_type (uidata->shell_view);
+
+	strategy = kolab_folder_metadata_ui_syncstrategy_get (uidata);
+	uidata->metadata->strategy = strategy;
+
  exit:
+
 	if (tmp_err != NULL) {
 		g_propagate_error (err, tmp_err);
 		ok = FALSE;
@@ -435,7 +608,10 @@ e_kolab_folder_metadata_ui_write_store (KolabFolderMetaUIData *uidata,
 	g_return_val_if_fail (E_IS_SHELL_VIEW (uidata->shell_view), FALSE);
 	g_return_val_if_fail (uidata->metadata != NULL, FALSE);
 
-	if (! uidata->changed)
+	if (uidata->shell_context != KOLAB_FOLDER_CONTEXT_EMAIL)
+		goto email_skip;
+
+	if (! (uidata->changed_metadata || uidata->changed_visibility))
 		return TRUE;
 
 	ok = e_kolab_plugin_util_ui_get_selected_store (uidata->shell_view,
@@ -449,11 +625,35 @@ e_kolab_folder_metadata_ui_write_store (KolabFolderMetaUIData *uidata,
 		g_warning ("%s()[%u] foldername change: stored '%s' vs. current '%s'",
 		           __func__, __LINE__, uidata->foldername, selected_path);
 
+	if (! uidata->metadata)
+		goto metadata_skip;
+
 	ok = camel_kolab_imapx_store_set_folder_type (kstore,
 	                                              uidata->foldername,
 	                                              uidata->metadata->foldertype,
 	                                              cancellable,
 	                                              &tmp_err);
+	if (! ok)
+		goto exit;
+
+ metadata_skip:
+
+	if (! uidata->changed_visibility)
+		goto exit;
+
+	ok = camel_kolab_imapx_store_set_show_all_folders (kstore,
+	                                                   uidata->metadata->show_all,
+	                                                   cancellable,
+	                                                   &tmp_err);
+ email_skip:
+
+	if (! ((uidata->shell_context == KOLAB_FOLDER_CONTEXT_CALENDAR) ||
+	       (uidata->shell_context == KOLAB_FOLDER_CONTEXT_CONTACT)))
+		goto exit;
+
+	ok = kolab_folder_metadata_ui_syncstrategy_set (uidata,
+	                                                cancellable,
+	                                                &tmp_err);
  exit:
 
 	if (tmp_err != NULL) {
