@@ -145,14 +145,14 @@ kolab_mail_handle_class_init (KolabMailHandleClass *klass)
 }
 
 /*----------------------------------------------------------------------------*/
-/* private util functions */
+/* local statics */
 
 
 static gboolean
-kolab_mail_handle_check_complete (const KolabMailHandle *self,
-                                  KolabFolderContextID context,
-                                  const gchar *function_name,
-                                  GError **err)
+mail_handle_check_complete (const KolabMailHandle *self,
+                            KolabFolderContextID context,
+                            const gchar *function_name,
+                            GError **err)
 {
 	KolabMailHandlePrivate *priv = NULL;
 	guint folder_context = 0;
@@ -351,10 +351,10 @@ kolab_mail_handle_convert_eds_to_kconvmail (KolabMailHandle *self,
 	                                                 KOLAB_MAIL_SUMMARY_UINT_FIELD_FOLDER_TYPE);
 	context = kolab_mail_summary_get_uint_field (priv->summary,
 	                                             KOLAB_MAIL_SUMMARY_UINT_FIELD_FOLDER_CONTEXT);
-	ok = kolab_mail_handle_check_complete (self,
-	                                       context,
-	                                       __func__,
-	                                       &tmp_err);
+	ok = mail_handle_check_complete (self,
+	                                 context,
+	                                 __func__,
+	                                 &tmp_err);
 	if (! ok) {
 		g_propagate_error (err, tmp_err);
 		return FALSE;
@@ -564,7 +564,7 @@ kolab_mail_handle_convert_kconvmail_to_eds (KolabMailHandle *self,
 			g_set_error (err,
 			             KOLAB_BACKEND_ERROR,
 			             KOLAB_BACKEND_ERROR_DATATYPE_KOLAB,
-			             "Conversion from Kolab contact type to Evolution failed for PIM Object, UID '%s', Folder '%s'",
+			             _("Conversion from Kolab contact type to Evolution failed for PIM Object, UID '%s', Folder '%s'"),
 			             priv->kolab_uid, priv->foldername);
 			return FALSE;
 		}
