@@ -125,10 +125,10 @@ read_task(I_task *i_task, xmlNodePtr task_node, GError **error)
 
 	/* cannot be properly converted back from evolution when startdate is missing, therefore => kolabstore */
 	if (i_task->incidence
-		&& i_task->incidence->recurrence
-		&& !i_task->incidence->start_date
-		&& (i_task->incidence->recurrence->recurrence_cycle == I_REC_CYCLE_YEARLY_MONTHDAY
-			|| i_task->incidence->recurrence->recurrence_cycle == I_REC_CYCLE_YEARLY_WEEKDAY)) {
+	    && i_task->incidence->recurrence
+	    && !i_task->incidence->start_date
+	    && (i_task->incidence->recurrence->recurrence_cycle == I_REC_CYCLE_YEARLY_MONTHDAY
+	        || i_task->incidence->recurrence->recurrence_cycle == I_REC_CYCLE_YEARLY_WEEKDAY)) {
 
 		g_free(i_task->incidence->recurrence);
 		i_task->incidence->recurrence = NULL;
@@ -209,8 +209,7 @@ handle_element_priority (gchar* name, gint *store, xmlNodePtr node)
 
 	value = xml_get_node_text(node);
 	g_debug ("**** handle_element_priority() node: %s - value: %s", node->name, value);
-	if (*store == -1)	/* already set? */
-	{
+	if (*store == -1) {	/* already set? */
 		errno = 0;
 		*store = priority_k_to_xkcal(strtol(value, &p, 10));
 		if (errno != 0 || *p != 0 || p == value)
